/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_BINDINGS_Q6_ASM_H__
#define __DT_BINDINGS_Q6_ASM_H__

#define	MSM_FRONTEND_DAI_MULTIMEDIA1	0
#define	MSM_FRONTEND_DAI_MULTIMEDIA2	1
#define	MSM_FRONTEND_DAI_MULTIMEDIA3	2
#define	MSM_FRONTEND_DAI_MULTIMEDIA4	3
#define	MSM_FRONTEND_DAI_MULTIMEDIA5	4
#define	MSM_FRONTEND_DAI_MULTIMEDIA6	5
#define	MSM_FRONTEND_DAI_MULTIMEDIA7	6
#define	MSM_FRONTEND_DAI_MULTIMEDIA8	7
#define	MSM_FRONTEND_DAI_MULTIMEDIA9	8
#define	MSM_FRONTEND_DAI_MULTIMEDIA10	9
#define	MSM_FRONTEND_DAI_MULTIMEDIA11	10
#define	MSM_FRONTEND_DAI_MULTIMEDIA12	11
#define	MSM_FRONTEND_DAI_MULTIMEDIA13	12
#define	MSM_FRONTEND_DAI_MULTIMEDIA14	13
#define	MSM_FRONTEND_DAI_MULTIMEDIA15	14
#define	MSM_FRONTEND_DAI_MULTIMEDIA16	15

#define Q6ASM_DAI_TX_RX	0
#define Q6ASM_DAI_TX	1
#define Q6ASM_DAI_RX	2

#endif /* __DT_BINDINGS_Q6_ASM_H__ */
